@@ -1,6 +1,6 @@
 /* mem.c --- memory for M32C simulator.
 
-Copyright (C) 2005-2017 Free Software Foundation, Inc.
+Copyright (C) 2005-2022 Free Software Foundation, Inc.
 Contributed by Red Hat, Inc.
 
 This file is part of the GNU simulators.
@@ -18,8 +18,10 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
+/* This must come before any other includes.  */
+#include "defs.h"
 
-#include "config.h"
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -27,9 +29,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
-#ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
-#endif
 #ifdef HAVE_TERMIOS_H
 #include <termios.h>
 #endif
@@ -294,7 +294,8 @@ mem_put_byte (int address, unsigned char value)
 	  }
 	else
 	  {
-	    write (m32c_console_ofd, &value, 1);
+	    if (write (m32c_console_ofd, &value, 1) != 1)
+	      printf ("write console failed: %s\n", strerror (errno));
 	  }
       }
       break;
@@ -367,11 +368,13 @@ mem_put_si (int address, unsigned long value)
 void
 mem_put_blk (int address, const void *bufptr, int nbytes)
 {
+  const unsigned char *buf = bufptr;
+
   S ("<=");
   if (enable_counting)
     mem_counters[1][1] += nbytes;
   while (nbytes--)
-    mem_put_byte (address++, *(const unsigned char *) bufptr++);
+    mem_put_byte (address++, *buf++);
   E ();
 }
 
@@ -443,7 +446,8 @@ mem_get_byte (int address)
     case 0x2ee:		/* m32c uart1 rx */
       {
 	char c;
-	read (m32c_console_ifd, &c, 1);
+	if (read (m32c_console_ifd, &c, 1) != 1)
+	  return 0;
 	if (m32c_console_ifd == 0 && c == 3)	/* Ctrl-C */
 	  {
 	    printf ("Ctrl-C!\n");
@@ -535,11 +539,13 @@ mem_get_si (int address)
 void
 mem_get_blk (int address, void *bufptr, int nbytes)
 {
+  char *buf = bufptr;
+
   S ("=>");
   if (enable_counting)
     mem_counters[0][1] += nbytes;
   while (nbytes--)
-    *(char *) bufptr++ = mem_get_byte (address++);
+    *buf++ = mem_get_byte (address++);
   E ();
 }
 
